//+++HDR+++
//======================================================================
//   This file is part of the BRAIN software framework.
//   Copyright (C) 2009 by CRS4, Pula, Italy.
//
//   For more information, visit the CRS4 Visual Computing Group
//   web pages at http://www.crs4.it/vic/
//
//   This file may be used under the terms of the GNU General Public
//   License as published by the Free Software Foundation and appearing
//   in the file LICENSE included in the packaging of this file.
//
//   CRS4 reserves all rights not expressly granted herein.
//  
//   This file is provided AS IS with NO WARRANTY OF ANY KIND, 
//   INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE.
//
//======================================================================
//---HDR---//
#include "bsAssignWidget.h"

#include <assert.h>
#include <q3listbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <q3whatsthis.h>
//Added by qt3to4:
#include <Q3VBoxLayout>
#include <Q3GridLayout>

#include <iostream>

bsAssignWidget::bsAssignWidget( QWidget* parent,  const char* name, bool modal, Qt::WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
        setName( "bsAssignWidget" );
    resize( 330, 244 ); 
    setCaption( tr( "Assign SubRegion Id"  ) );
    setSizeGripEnabled( TRUE );
    grid = new Q3GridLayout( this ); 
    grid->setSpacing( 6 );
    grid->setMargin( 11 );

    vbox = new Q3VBoxLayout; 
    vbox->setSpacing( 6 );
    vbox->setMargin( 0 );

    m_ok_button = new QPushButton( this, "m_ok_button" );
    m_ok_button->setText( tr( "&OK"  ) );
    m_ok_button->setAutoDefault( TRUE );
    m_ok_button->setDefault( TRUE );
    vbox->addWidget( m_ok_button );

    m_cancel_button = new QPushButton( this, "m_cancel_button" );
    m_cancel_button->setText( tr( "&Cancel"  ) );
    m_cancel_button->setAutoDefault( TRUE );
    vbox->addWidget( m_cancel_button );
    QSpacerItem* spacer = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
    vbox->addItem( spacer );

    grid->addLayout( vbox, 0, 1 );

    m_list_box = new Q3ListBox( this, "m_list_box" );

    grid->addWidget( m_list_box, 0, 0 );

    // signals and slots connections
    connect( m_ok_button, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( m_cancel_button, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

/*  
 *  Destroys the object and frees any allocated resources
 */
bsAssignWidget::~bsAssignWidget()
{
    // no need to delete child widgets, Qt does it all for us
}


void bsAssignWidget::setItem(const char* label){
    assert(label);
    m_list_box->insertItem( tr(label) );
}

void bsAssignWidget::setCurrentItem(int id) {
    m_list_box->setCurrentItem(id);
}

void bsAssignWidget::accept(){
    QDialog::accept();
    setResult(m_list_box->currentItem());
}

void bsAssignWidget::reject(){
    QDialog::reject();
    setResult(-1);
}


