//+++HDR+++
//======================================================================
//   This file is part of the BRAIN software framework.
//   Copyright (C) 2009 by CRS4, Pula, Italy.
//
//   For more information, visit the CRS4 Visual Computing Group
//   web pages at http://www.crs4.it/vic/
//
//   This file may be used under the terms of the GNU General Public
//   License as published by the Free Software Foundation and appearing
//   in the file LICENSE included in the packaging of this file.
//
//   CRS4 reserves all rights not expressly granted herein.
//  
//   This file is provided AS IS with NO WARRANTY OF ANY KIND, 
//   INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE.
//
//======================================================================
//---HDR---//
#include "bsCuttingBoxWidgetImpl.h"

#include <iostream>
#include <qstring.h>
#include <qslider.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>

/* 
 *  Constructs a bsCuttingBoxWidgetImpl which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
bsCuttingBoxWidgetImpl::bsCuttingBoxWidgetImpl( QWidget* parent,  const char* name, Qt::WFlags fl )
    : bsCuttingBoxWidget( parent, name, fl ) {
  m_width = 0.0f;
  m_length = 0.0f;
  m_height = 0.0f;

  connect(m_w, SIGNAL(returnPressed()), this, SLOT(emit_wlhChanged()));
  connect(m_l, SIGNAL(returnPressed()), this, SLOT(emit_wlhChanged()));
  connect(m_h, SIGNAL(returnPressed()), this, SLOT(emit_wlhChanged()));

  connect(m_slider_i,SIGNAL(valueChanged(int)), this, SLOT(emit_slidersChanged(int)));
  connect(m_slider_j,SIGNAL(valueChanged(int)), this, SLOT(emit_slidersChanged(int)));
  connect(m_slider_k,SIGNAL(valueChanged(int)), this, SLOT(emit_slidersChanged(int)));

  connect(m_reset_button, SIGNAL(clicked()), this, SLOT(emit_resetClicked()));
  connect(m_reset_button, SIGNAL(clicked()), this, SLOT(resetSliders()));

}


/*  
 *  Destroys the object and frees any allocated resources
 */
bsCuttingBoxWidgetImpl::~bsCuttingBoxWidgetImpl()
{
    // no need to delete child widgets, Qt does it all for us
}

void bsCuttingBoxWidgetImpl::setW(const float val) {
  QString str;
  str.sprintf("%.0f", val);
  m_w->setText(str);
  m_width=val;
}

void bsCuttingBoxWidgetImpl::setL(const float val) {
  QString str;
  str.sprintf("%.0f", val);
  m_l->setText(str);
  m_length=val;
}

void bsCuttingBoxWidgetImpl::setH(const float val) {
  QString str;
  str.sprintf("%.0f", val);
  m_h->setText(str);
  m_height=val;
}


void bsCuttingBoxWidgetImpl::emit_wlhChanged() {
  bool ok;
  float val;
  val = m_w->text().toFloat(&ok);
  setW(ok ? val : m_width);

  val = m_l->text().toFloat(&ok);
  setL(ok ? val : m_length);

  val = m_h->text().toFloat(&ok);
  setH(ok ? val : m_height);
  
  emit wlhChanged();
}


void bsCuttingBoxWidgetImpl::emit_resetClicked() {
  emit resetClicked();
}

void bsCuttingBoxWidgetImpl::emit_slidersChanged(int v) {
  if(v!=0) {
    emit slidersChanged();
  }
}

void bsCuttingBoxWidgetImpl::resetSliders() {
  m_slider_i->setValue(0);
  m_slider_j->setValue(0);
  m_slider_k->setValue(0);
  enableSliders(true);
}

void bsCuttingBoxWidgetImpl::enableSliders(bool b) {
  m_slider_i->setEnabled(b);
  m_slider_j->setEnabled(b);
  m_slider_k->setEnabled(b);
  if(b) {
    m_slider_i->show();
    m_slider_j->show();
    m_slider_k->show();
    m_label_i->show();
    m_label_j->show();
    m_label_k->show();
  } else {
    m_slider_i->hide();
    m_slider_j->hide();
    m_slider_k->hide();
    m_label_i->hide();
    m_label_j->hide();
    m_label_k->hide();
  }
}


float bsCuttingBoxWidgetImpl::normalizeSlider(int i) const {
  QSlider* tmp=NULL;
  switch (i) {
  case 0: tmp=m_slider_i;
    break;
  case 1: tmp=m_slider_j;
    break;
  case 2: tmp=m_slider_k;
    break;
  }
  float maxV=tmp->maxValue();
  float v = tmp->value();
  return (v/maxV);
}


void bsCuttingBoxWidgetImpl::setSliders(float vi, float vj, float vk) {
  m_slider_i->setValue(int(vi*m_slider_i->maxValue()));
  m_slider_j->setValue(int(vj*m_slider_j->maxValue()));
  m_slider_k->setValue(int(vk*m_slider_k->maxValue()));
}
