//+++HDR+++
//======================================================================
//   This file is part of the BRAIN software framework.
//   Copyright (C) 2009 by CRS4, Pula, Italy.
//
//   For more information, visit the CRS4 Visual Computing Group
//   web pages at http://www.crs4.it/vic/
//
//   This file may be used under the terms of the GNU General Public
//   License as published by the Free Software Foundation and appearing
//   in the file LICENSE included in the packaging of this file.
//
//   CRS4 reserves all rights not expressly granted herein.
//  
//   This file is provided AS IS with NO WARRANTY OF ANY KIND, 
//   INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE.
//
//======================================================================
//---HDR---//
#include "bsMainWindow.h"
#include "bsEditor2D.h"
#include "bsMiniFastEditor3D.h"
#include "bsFastEditor3D.h"
#include "bsHQIsoEditor3D.h"
#include "bsSectionedBrain.h"
#include "bsSection.h"
#include "bsDocument.h"
#include "bsChooseOptions.h"
#include "bsViewer.h"

#include <QImage>

#include "qtCommandHistory.h"

#include <qpixmap.h>
#include <qimage.h>
#include <q3toolbar.h>
#include <qtoolbutton.h>
#include <q3popupmenu.h>
#include <qmenubar.h>
#include <qlayout.h>
#include <qnamespace.h>
#include <qfile.h>
#include <q3filedialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <q3accel.h>
#include <q3textstream.h>
#include <qpainter.h>
#include <q3paintdevicemetrics.h>
#include <q3whatsthis.h>
#include <qtabwidget.h>
#include <q3textview.h>

#include "qtCommandHistoryBrowser.h"
//Added by qt3to4:
#include <Q3GridLayout>
#include <Q3Frame>
#include <QCloseEvent>

#include "Icons/filesave.xpm"
#include "Icons/fileopen.xpm"
#include "Icons/filehist.xpm"
#include "Icons/undo.xpm"
#include "Icons/redo.xpm"
#include "Icons/crs4_m.xpm"

#define Max(a,b)  ((a > b) ? (a) : (b))
#define Min(a,b)  ((a > b) ? (b) : (a))

const char * fileOpenText = "<img source=\"fileopen\"> "
"Click this button to open a <em>new XML file</em>. <br><br>"
"You can also select the <b>Open command</b> from the File menu.";
const char * fileSaveText = "Click this button to save the file in XML format"
"You will be prompted for a file name.\n\n"
"You can also select the Save command from the File menu.\n\n";


bsMainWindow::bsMainWindow(qtCommandHistory* cmd_processor)
    : Q3MainWindow( 0, "Main Window", Qt::WDestructiveClose ) {
  
  assert(cmd_processor);
  m_cmd_processor=cmd_processor;

  //    m_cmd_processor = (cmd_processor) ? (cmd_processor) : new qtCommandHistory(-1);

    m_saveOK=false;

    int id;
    QPixmap openIcon, saveIcon, histIcon, undoIcon, redoIcon;

    m_file_tools = new Q3ToolBar( this, "file operations" );
    m_file_tools->setLabel( tr( "File Operations" ) );

    openIcon = QPixmap( fileopen );
    QToolButton* fileOpen
	= new QToolButton( openIcon, "Open File", QString::null,
			   this, SLOT(loadXml()), m_file_tools, "open file" );

    saveIcon = QPixmap( filesave );
    m_file_save_xml
	= new QToolButton( saveIcon, "Save File", QString::null,
			   this, SLOT(saveXml()), m_file_tools, "save file" );

    histIcon = QPixmap( filehist );
    QToolButton* fileHist
	= new QToolButton( histIcon, "History", QString::null,
			   this, SLOT(showHist()), m_file_tools, "history" );
    fileHist->setToggleButton(true);

    undoIcon = QPixmap( undoPixmap );
    m_undo_button
	= new QToolButton( undoIcon, "Undo", QString::null,
			   this, SLOT(undo()), m_file_tools, "undoButton" );

    redoIcon = QPixmap( redoPixmap );
    m_redo_button
	= new QToolButton( redoIcon, "Redo", QString::null,
			   this, SLOT(redo()), m_file_tools, "redoButton" );

    (void)Q3WhatsThis::whatsThisButton( m_file_tools );

    Q3WhatsThis::add( fileOpen, fileOpenText );
    Q3MimeSourceFactory::defaultFactory()->setPixmap( "fileopen", openIcon );
    Q3WhatsThis::add( m_file_save_xml, fileSaveText );

    m_file = new Q3PopupMenu( this );
    menuBar()->insertItem( "&File", m_file );

    id = m_file->insertItem( "&Load Settings",
			   this, SLOT(loadSettings()));
    id = m_file->insertItem( "&Save Settings",
			   this, SLOT(saveSettings()));
    m_file->insertSeparator();

    id = m_file->insertItem( "&Import",
			     this, SLOT(loadInside()), Qt::CTRL+Qt::Key_I );
    id = m_file->insertItem( openIcon, "&Open",
			   this, SLOT(loadXml()), Qt::CTRL+Qt::Key_O );
    m_file->setWhatsThis( id, fileOpenText );

    m_id_save_xml = m_file->insertItem( saveIcon, "&Save",
			   this, SLOT(saveXml()), Qt::CTRL+Qt::Key_S );
    m_file->setWhatsThis( m_id_save, fileSaveText );

    m_id_save = m_file->insertItem( "&Export",
			   this, SLOT(saveInside()), Qt::CTRL+Qt::Key_E );

    m_file->insertSeparator();
    m_id_pixmap = m_file->insertItem( "&Pixmap",
			   this, SLOT(savePixmap()), Qt::CTRL+Qt::Key_X );

    m_file->insertSeparator();
    m_file->insertItem( "&Quit", qApp, SLOT( closeAllWindows() ), Qt::CTRL+Qt::Key_Q );

    
    m_edit = new Q3PopupMenu( this );
    menuBar()->insertItem( "&Edit", m_edit );
    m_id_undo = m_edit->insertItem( undoIcon, "&Undo",  this, SLOT(undo()), Qt::CTRL+Qt::Key_Z );
    m_id_redo = m_edit->insertItem( redoIcon, "&Redo",  this, SLOT(redo()), Qt::CTRL+Qt::Key_T );
    m_id_hist = m_edit->insertItem( histIcon, "&History", this, SLOT(showHist()), Qt::CTRL+Qt::Key_H );
    m_id_hist_flag=FALSE;
    m_edit->setCheckable(TRUE);

    m_options = new Q3PopupMenu( this );
    menuBar()->insertItem( "&Options", m_options );
    m_id_sub = m_options->insertItem("&Subregions...",  this, SLOT(showSubregionOptions()), Qt::CTRL+Qt::Key_R );    
    m_id_neuro = m_options->insertItem("&Neurons...",  this, SLOT(showNeuronOptions()), Qt::CTRL+Qt::Key_N );    

    m_hb = new qtCommandHistoryBrowser();
    m_hb->setHistory(m_cmd_processor);

    connect( m_cmd_processor, SIGNAL(historyChanged(qtCommandHistory*)), this, SLOT(updateMainWindow()));
    updateMainWindow();

    Q3PopupMenu * help = new Q3PopupMenu( this );
    menuBar()->insertSeparator();
    menuBar()->insertItem( "&Help", help );

    QPixmap logoIcon(crs4Pixmap);
    menuBar()->insertItem(logoIcon);

    help->insertItem( "&About", this, SLOT(about()), Qt::Key_F1 );
    help->insertItem( "About &CRS4", this, SLOT(aboutCRS4()) );
    help->insertItem( "About &Parma", this, SLOT(aboutParma()) );
    help->insertSeparator();
    help->insertItem( "What's &This", this, SLOT(whatsThis()), Qt::SHIFT+Qt::Key_F1 );

    
    m_tab_widget = new QTabWidget( this, "m_tab_widget" );

    // TAB_1
    Q3Frame* desk_tab = new Q3Frame(m_tab_widget,"desk_tab");
    Q3GridLayout* tabLayout = new Q3GridLayout(desk_tab,2,2); 
    tabLayout->setSpacing( 6 );
    tabLayout->setMargin( 11 );

    m_text_info = new Q3TextView( desk_tab, "m_text_info" );
    m_text_info->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, 
					     (QSizePolicy::SizeType)5, 
					     m_text_info->sizePolicy().hasHeightForWidth() ) );
    m_text_info->setMinimumSize( QSize( 220, 0 ) );
    m_text_info->setMaximumSize( QSize( 240, 32767 ) );

    tabLayout->addWidget( m_text_info, 0, 0 );

    m_fast_viewer3D = new bsMiniFastEditor3D(desk_tab, "m_fast_viewer3D", 0, m_cmd_processor);
    m_fast_viewer3D->setMinimumSize( QSize( 220, 240 ) );
    m_fast_viewer3D->setMaximumSize( QSize( 240, 240 ) );
    m_fast_viewer3D->setEnableSubRegions(false);
    m_fast_viewer3D->setEnableNeuronTypes(false);
    m_fast_viewer3D->setEnableSelectedRenderer(true);
    tabLayout->addWidget( m_fast_viewer3D, 1, 0 );

    m_editor2D = new bsEditor2D(desk_tab, "m_editor2D", 0, m_cmd_processor);
    m_editor2D->setMinimumSize( QSize( 710, 500 ) );
    tabLayout->addMultiCellWidget( m_editor2D, 0, 1, 1, 1 );
    m_tab_widget->insertTab( desk_tab, tr( "Section Editor" ) );

    connect( m_editor2D, SIGNAL(currentSectionChanged(int)), m_fast_viewer3D, SLOT(setSelectedRendererIndex(int)) );

    //TAB_2
    m_editor3D = new bsFastEditor3D(m_tab_widget, "m_editor3D", 0, m_cmd_processor);
    m_editor3D->setEnableSubRegions(false);
    m_editor3D->setEnableNeuronTypes(false);
    m_tab_widget->insertTab(m_editor3D, tr("3D Viewer"));

    //TAB_3
    m_hq_iso_viewer3D = new bsHQIsoEditor3D(m_tab_widget, "m_hq_iso_viewer3D", 0, m_cmd_processor);
    m_hq_iso_viewer3D->setEnableSubRegions(false);
    m_hq_iso_viewer3D->setEnableNeuronTypes(false);
    m_tab_widget->insertTab(m_hq_iso_viewer3D, tr("HQ Iso Viewer"));


    // General
    setCentralWidget(m_tab_widget);

    m_document = new bsDocument(this, "document", m_cmd_processor);
    connect(m_document, SIGNAL(newBrain2D(bool)), this, SLOT(enableActions()));
    connect(m_document, SIGNAL(brain2DChanged(int)), this, SLOT(updateTextInfo()));
    connect(m_document, SIGNAL(regionOptionsChanged()), this, SLOT(updateTextInfo()));
    connect(m_document, SIGNAL(neuronTypeOptionsChanged()), this, SLOT(updateTextInfo()));

    // Maybe this is not useful!
    // connect(m_tab_widget, SIGNAL(currentChanged(QWidget *)), this, SLOT(updateCurrentViewer()));
    

    m_editor2D->setDocument(m_document);
    m_editor3D->setDocument(m_document);
    m_fast_viewer3D->setDocument(m_document);
    m_hq_iso_viewer3D->setDocument(m_document);

    updateTextInfo();

    m_choose_subregion_options = new bsChooseOptions(0, "Subregion Options", true);
    m_choose_neuron_options = new bsChooseOptions(0, "Neuron Type Options", true, 0, true);

    m_cmd_processor->clear();

    enableActions();
    statusBar()->message( "Ready", 2000 );
}


//bsMainWindow::~bsMainWindow() {
//}


// Private Slots

void bsMainWindow::loadInside() {

    bool load_flag=true;
    if (m_document->getBrain2D()) {
	if (m_document->brain2DModified()) {
	    switch( QMessageBox::information( this, "bs Viewer",
					      "A visualization is running...\n"
					      "Save before load?",
					      "Yes", "No", "Cancel",
					      0, 2 ) ) {
	    case 0:
		saveXml();
		if(!m_saveOK) load_flag=false;
		break;
	    case 1:
		break;
	    case 2:
		load_flag=false;
		break;
	    }	

	} else {
	    switch( QMessageBox::information( this, "bs Viewer",
					      "A visualization is running...\n"
					      "Load a new one anyway?",
					      "Yes", "No",
					      0, 1 ) ) {
	    case 0:
		break;
	    case 1:
		load_flag=false;
		break;
	    }	
	}
    }
    
    if (load_flag) {
#ifndef NDEBUG
      QString dirStart;
      //QString dirStart("/xlv0/tmp/Data");
#else
	QString dirStart;
#endif
	QString fn = Q3FileDialog::getOpenFileName( dirStart,
						   "Inside files (*.dat);;All files (*)",
						   this);
	if ( !fn.isEmpty() ) {
	    loadInside( fn );
	}
	else
	    statusBar()->message( "Loading aborted", 2000 );    
    }
}


void bsMainWindow::loadInside( const char *fileName ) {
    m_document->loadInside(fileName);
    if (m_document->lastError() != 0){
	QMessageBox::information( this,"ERROR", m_document->lastMessage());
    } 
    statusBar()->message(m_document->lastMessage(), 6000 );
}


void bsMainWindow::loadXml() {
    bool load_flag=true;
    if (m_document->getBrain2D()) {
	if (m_document->brain2DModified()) {
	    switch( QMessageBox::information( this, "bs Viewer",
					      "A visualization is running...\n"
					      "Save before load?",
					      "Yes", "No", "Cancel",
					      0, 2 ) ) {
	    case 0:
		saveXml();
		if(!m_saveOK) load_flag=false;
		break;
	    case 1:
		break;
	    case 2:
		load_flag=false;
		break;
	    }	

	} else {
	    switch( QMessageBox::information( this, "bs Viewer",
					      "A visualization is running...\n"
					      "Load a new one anyway?",
					      "Yes", "No",
					      0, 1 ) ) {
	    case 0:
		break;
	    case 1:
		load_flag=false;
		break;
	    }	
	}
    }
    
    if (load_flag) {
#ifndef NDEBUG
      QString dirStart;
      //QString dirStart("/xlv0/tmp/Data");
#else
	QString dirStart;
#endif
	QString fn = Q3FileDialog::getOpenFileName( dirStart,
						   "XML files (*.xml);;All files (*)",
						   this);
	if ( !fn.isEmpty() ) {
	    loadXml( fn );
	}
	else
	    statusBar()->message( "Loading aborted", 2000 );    
    }
}


void bsMainWindow::loadXml( const char *fileName ) {
    m_document->loadXml(fileName);
    if (m_document->lastError() != 0){
	QMessageBox::information( this,"ERROR", m_document->lastMessage());
    } 
    statusBar()->message(m_document->lastMessage(), 6000 );
}


void bsMainWindow::saveInside()
{
    QString fn = Q3FileDialog::getSaveFileName( QString::null,
					       "Inside files (*.dat);;All files (*)",
					       this);
    if ( !fn.isEmpty() ) {
	m_saveOK=true;
	saveInside( fn );
    }
    else {
	statusBar()->message( "Save process aborted", 2000 );
	m_saveOK=false;
    }
}

void bsMainWindow::saveInside( const char *fileName ) {
    m_document->saveInside(fileName);
    if (m_document->lastError() != 0){
	QMessageBox::information( this,"ERROR", m_document->lastMessage());
	m_saveOK=false;
    } 
    statusBar()->message(m_document->lastMessage(), 6000 );
}


void bsMainWindow::saveXml()
{
    QString fn = Q3FileDialog::getSaveFileName( QString::null,
					       "XML files (*.xml);;All files (*)",
					       this);
    if ( !fn.isEmpty() ) {
	m_saveOK=true;
	saveXml( fn );
    }
    else {
	statusBar()->message( "Save process aborted", 2000 );
	m_saveOK=false;
    }
}

void bsMainWindow::saveXml( const char *fileName ) {
    m_document->saveXml(fileName);
    if (m_document->lastError() != 0){
	QMessageBox::information( this,"ERROR", m_document->lastMessage());
	m_saveOK=false;
    } 
    statusBar()->message(m_document->lastMessage(), 6000 );
}


void bsMainWindow::about()
{
    QMessageBox::about( this, "BRAIN",
			"Interactive 3-D system for reconstruction and visualization "
                        "of primates cerebral cortex.");
}

void bsMainWindow::aboutCRS4()
{
    QMessageBox::about( this, "About CRS4",
			"CRS4 - Centro di Ricerca, Sviluppo e\n"
			"Studi Superiori in Sardegna\n"
			"Visual Computing Group\n"
			"Sardegna Ricerche Ed. 1\n"
			"P.O. Box 25\n"
			"I-09010 Pula (CA) - Italy\n"
			"Tel: +39 070 9250.212\n"
			"Fax: +39 070 9250.216\n"
			"http://www.crs4.it/vic");
}


void bsMainWindow::aboutParma()
{
    QMessageBox::about( this, "About Parma",
			"University of Parma\n"
			"Department of Neurosciences\n"
			"Section of Physiology\n"
			"Via Volturno, 39\n"
			"I-43100 Parma (PR) - Italy\n"
			"Tel: +39 0521 903881\n"
			"Fax: +39 0521 903900\n"
			"http://www.unipr.it/");
}

void bsMainWindow::closeEvent( QCloseEvent* ce )
{
    if ( !m_document->getBrain2D()) {
        ce->accept();
        return;
    }
    
    if (m_document->brain2DModified()) {
	switch( QMessageBox::information( this, "bs Viewer",
					  "Document has been modified.\n"
					  "Save before exit?",
					  "Yes", "No", "Cancel",
					  0, 2) ) {
	case 0:
	    saveXml();
	    if(m_saveOK){
		ce->accept();
	    } else {
		ce->ignore();
	    }
	    break;
	case 1:
	    ce->accept();
	    break;
	case 2:
	    ce->ignore();
	    break;
	}
    } else {
	switch( QMessageBox::information( this, "bs Viewer",
					  "Exit. Are you sure?",
					  "Yes", "No",
					  0, 1 ) ) {
	case 0:
	    ce->accept();
	    break;
	case 1:
	    ce->ignore();
	    break;
	}
    }
}

void  bsMainWindow::updateMainWindow(){
    m_edit->setItemEnabled(m_id_undo, m_cmd_processor->doneCount() > 0);
    m_undo_button->setEnabled(m_cmd_processor->doneCount() > 0);
    m_edit->setItemEnabled(m_id_redo, m_cmd_processor->undoneCount() > 0);
    m_redo_button->setEnabled(m_cmd_processor->undoneCount() > 0);
}

void bsMainWindow::enableActions(){
    m_file->setItemEnabled(m_id_save_xml, m_document->hasBrain2D());
    m_file->setItemEnabled(m_id_save, m_document->hasBrain2D());
    m_file_save_xml->setEnabled(m_document->hasBrain2D());
    m_options->setItemEnabled(m_id_sub, m_document->hasBrain2D());
    m_options->setItemEnabled(m_id_neuro, m_document->hasBrain2D());
    updateTextInfo();
}

void bsMainWindow::showHist()
{
    m_id_hist_flag = ! m_id_hist_flag;
    m_edit->setItemChecked( m_id_hist, m_id_hist_flag );
    if (m_id_hist_flag) {
	 m_hb->show();
    } else {
	 m_hb->hide();
    }
}



// Command

void bsMainWindow::undo() {
  if (m_cmd_processor->doneCount() > 0) {
    m_cmd_processor->undo();
  }
}


void bsMainWindow::showNeuronOptions() {
  if (m_document->hasBrain2D()) {
    m_choose_neuron_options->clear();
    bsSectionedBrain* brain = m_document->getBrain2D();
    for (int tp=0; tp <= brain->maxNeuronType(); tp++) {
      sl::color3f c = brain->neuronTypeColorItem(tp);
      QColor qc = QColor((int) (c[0]*255.0f), (int) (c[1]*255.0f), (int) (c[2]*255.0f)); 
      m_choose_neuron_options->addItem(qc,brain->neuronTypeLabelItem(tp).c_str(),
				       brain->neuronTypeAligned(tp),
				       brain->neuronTypeEnabled(tp),
				       brain->neuronTypeDimension(tp));
    }
    int result =  m_choose_neuron_options->exec();
    if ( result == QDialog::Accepted ) {
      m_document->setNeuronTypeOptions(m_choose_neuron_options);
      m_document->setNeuronSize((float) m_choose_neuron_options->getDimension());
    }
  }
}


void bsMainWindow::showSubregionOptions() {
  if (m_document->hasBrain2D()) {
    m_choose_subregion_options->clear();
    bsSectionedBrain* brain = m_document->getBrain2D();
    for (int tp=0; tp < brain->regionLabelCount(); tp++) {
      sl::color3f c = brain->regionColorItem(tp);
      QColor qc = QColor((int) (c[0]*255.0f), (int) (c[1]*255.0f), (int) (c[2]*255.0f)); 
      m_choose_subregion_options->addItem(qc,brain->regionLabelItem(tp).c_str());
    }
    int result =  m_choose_subregion_options->exec();
    if ( result == QDialog::Accepted ) {
      m_document->setRegionOptions(m_choose_subregion_options);
      m_document->setBorderSize((float) m_choose_subregion_options->getDimension());
    }
  }
}


void bsMainWindow::redo() {
  if (m_cmd_processor->undoneCount() > 0) {
    m_cmd_processor->redo();
  }
}


void bsMainWindow::updateTextInfo() {
  QString s;
  s= tr( "<qt>" );
  s += tr( "<center><h2>BRAIN</h2></center><p>" );
  if (m_document->hasBrain2D()) {
    bsSectionedBrain* brain = m_document->getBrain2D();
    sl::aabox3f box = brain->boundingBox();
    
    float w = box[1][0]-box[0][0];
    float l = box[1][1]-box[0][1];
    float h = box[1][2]-box[0][2];

    QString tmp;

    tmp.sprintf("Number of sections: %d<p>",brain->sectionCount());
    s += tmp;

    tmp = tr ( "<b>Dimensions [micron]:</b><br>" );
    s += tmp;

    tmp.sprintf("<b>w:</b> %6.0f <b>h:</b> %6.0f <b>l:</b> %6.0f<p>",w,h,l);
    s += tmp;

    tmp.sprintf("<b>Volume:</b> %.3e mm^3<p>",brain->brainVolume()*1.0e-9);
    s += tmp;

    QString colorA( "\"#FFFFFF\"" );
    QString colorB( "\"#DDDDDD\"" );
    QString bgcolor;
    QString color;

    s += tr( "<b>Subregions:</b>" );
    s += tr( "<center><table border=\"1\"><tr>,<th>Color</th><th>Type</th><th>Volume [mm^3]</th></tr>" );
    {
      for (int tp=0; tp < brain->regionLabelCount(); tp++) {
	sl::color3f c = brain->regionColorItem(tp);
	color.sprintf("\"#%02X%02X%02X\"",(int)(c[0]*255.0f),(int)(c[1]*255.0f),(int)(c[2]*255.0f));
	bgcolor = (tp%2) ? colorB : colorA;
	tmp.sprintf("<tr bgcolor=%s><td bgcolor=%s> </td><td>%s</td><td>%.3e</td></tr>",
		    bgcolor.latin1(),color.latin1(),brain->regionLabelItem(tp).c_str(),
		    brain->subRegionVolume(tp)*1.0e-9);
	s += tmp;
      }
    }
    s += tr( "</table></center><br>" );

    s += tr( "<b>Neural tracers:</b>" );
    s += tr( "<center><table border=\"1\"><tr>,<th>Color</th><th>Type</th><th>Count</th></tr>" );
    {
      for (int tp=0; tp <= brain->maxNeuronType(); tp++) {
	float result = 0.0f;
	for (int ns=0; ns<brain->sectionCount(); ns++) {
	  if ( ! brain->sectionItem(ns)->ghostSection() ) {
	    result += brain->sectionItem(ns)->neuronCount(tp);
	  }
	}
	sl::color3f c = brain->neuronTypeColorItem(tp);
	color.sprintf("\"#%02X%02X%02X\"",(int)(c[0]*255.0f),(int)(c[1]*255.0f),(int)(c[2]*255.0f));
	bgcolor = (tp%2) ? colorB : colorA;
	tmp.sprintf("<tr bgcolor=%s><td bgcolor=%s> </td><td>%s</td><td align=\"right\">%6.0f</td></tr>",
		    bgcolor.latin1(),color.latin1(),brain->neuronTypeLabelItem(tp).c_str(),result);
	s += tmp;
      }
    }
    s += tr( "</table></center>" );
  } else {
  }
  s += tr( "</qt>" );

  m_text_info->setText(s);
}


void bsMainWindow::savePixmap()
{
    QString fn = Q3FileDialog::getSaveFileName( QString::null,
					       "BMP (*.bmp)",
					       this);
    if ( !fn.isEmpty() ) {
	savePixmap( fn );
    }
}

QImage bsMainWindow::getPixmapLegend() {
  
  int w=200;
  int h=300;
  QPixmap img(w,h);
  if (m_document->hasBrain2D()) {
    int init_posx1=4;
    int init_posx2=120;
    int init_posy=24;
    int delta=16;
    int box_size=10;
    int space=13;
    
    bsSectionedBrain* brain = m_document->getBrain2D();
    w=260+space;
    h=(Max(brain->regionLabelCount(),brain->regionLabelCount())+5)*delta;

    img.resize(w,h);
    img.fill(Qt::white);

    QPainter paint;
    paint.begin(&img);
    QFont font("Helvetica",12);
    paint.setFont(font);
    
    paint.setPen(Qt::black);
    paint.drawRect(0,0,img.width(),img.height());
    paint.drawText(100,init_posy,"LEGEND");
    paint.drawText(init_posx1,init_posy+delta,"Regions");
    
    for (int tp=0; tp < brain->regionLabelCount(); tp++) {
      sl::color3f c = brain->regionColorItem(tp);
      QColor col((int)(c[0]*255.0f),(int)(c[1]*255.0f),(int)(c[2]*255.0f));
      QString s = brain->regionLabelItem(tp).c_str();
      paint.setBrush(col);
      int y=init_posy+(tp+2)*delta;
      paint.drawRect(init_posx1,y-box_size,box_size,box_size);
      paint.drawText(init_posx1+box_size+space,y,s);
    }
    

    paint.drawText(init_posx2,init_posy+delta,"Neurons");
    for (int tp=0; tp < brain->neuronTypeLabelCount(); tp++) {
      sl::color3f c = brain->neuronTypeColorItem(tp);
      QColor col((int)(c[0]*255.0f),(int)(c[1]*255.0f),(int)(c[2]*255.0f));
      QString s = brain->neuronTypeLabelItem(tp).c_str();
      paint.setBrush(col);
      int y=init_posy+(tp+2)*delta;
      paint.drawRect(init_posx2,y-box_size,box_size,box_size);
      paint.drawText(init_posx2+box_size+space,y,s);
    }
    QString sect;
    int n_sect=m_editor2D->getCurrentSectionIndex();
    sect.sprintf("Current section: %3d",n_sect);
    paint.drawText(init_posx1,img.height()-delta,sect);
    paint.end();
  }
  QImage im=img.convertToImage(); 
  return im;
}

void bsMainWindow::drawImage(QImage* bkg, int x, int y, QImage* img) {
  if(bkg->valid(x,y)) {
    int xEnd=Min(x+img->width(),bkg->width());
    int yEnd=Min(y+img->height(),bkg->height());
    for (int yy=y; yy < yEnd; yy++) {
      for (int xx=x; xx < xEnd; xx++) {
	bkg->setPixel(xx,yy,img->pixel(xx-x,yy-y));
      }
    }
  }
}


void bsMainWindow::savePixmap( const char *fileName ) {
  QImage img;
  raise();
  switch (m_tab_widget->currentPageIndex()) {
  case 0: {
    bsViewer* v =  m_editor2D->getViewer();
    QImage img2D=v->getGLPixmap();
    int w2D=img2D.width();
    int h2D=img2D.height();
    bsViewer* v3D =  m_fast_viewer3D->getViewer();
    QImage img3D=v3D->getGLPixmap();
    int w3D=img3D.width();
    int h3D=img3D.height();
    QImage imgLegend=getPixmapLegend();
    int wLegend=imgLegend.width();
    int hLegend=imgLegend.height();
    int w = Max(wLegend,w3D);
    int h = Max(hLegend+h3D,h2D);

    img.create(w+w2D+2,h,32);
    img.fill(qRgba(255,255,255,255));
    drawImage(&img,0,h-h3D,&img3D);
    drawImage(&img,w+2,h-h2D,&img2D);
    drawImage(&img,0,0,&imgLegend);
  }
  break;
  case 1: {
    bsViewer* v =  m_editor3D->getViewer();
    img = v->getGLPixmap();
  }
  break;
  case 2: {
    bsViewer* v =  m_hq_iso_viewer3D->getViewer();
    img = v->getGLPixmap();
  }
  break;
  }

  if (!img.save(fileName)){
    QMessageBox::information( this,"ERROR", "File NOT saved!");
    statusBar()->message("File NOT saved!", 6000 );    
  } else {
    statusBar()->message("File saved!", 6000 );    
  }
}


void bsMainWindow::updateCurrentViewer() {
  switch (m_tab_widget->currentPageIndex()) {
  case 0: 
    {
      m_editor2D->updateViewer();
      m_fast_viewer3D->updateViewer();
    }
    break;
  case 1: 
    {
      m_editor3D->updateViewer();
    }
    break;
  case 2: 
    {
      m_hq_iso_viewer3D->updateViewer();
    }
    break;
  }
}


// File of Setting Parameters

void bsMainWindow::saveSettings()
{
    QString fn = Q3FileDialog::getSaveFileName( QString::null,
					       "BSP files (*.bsp);;All files (*)",
					       this);
    if ( !fn.isEmpty() ) {
	m_saveOK=true;
	saveSettings( fn );
    }
    else {
	statusBar()->message( "Save process aborted", 2000 );
	m_saveOK=false;
    }
}

void bsMainWindow::saveSettings( const char *fileName ) {
    m_document->saveSettings(fileName);
    if (m_document->lastError() != 0){
	QMessageBox::information( this,"ERROR", m_document->lastMessage());
	m_saveOK=false;
    } 
    statusBar()->message(m_document->lastMessage(), 6000 );
}


void bsMainWindow::loadSettings() {
#ifndef NDEBUG
      QString dirStart;
      //QString dirStart("/xlv0/tmp/Data");
#else
	QString dirStart;
#endif
	QString fn = Q3FileDialog::getOpenFileName( dirStart,
						   "BSP files (*.bsp);;All files (*)",
						   this);
	if ( !fn.isEmpty() ) {
	    loadSettings( fn );
	}
	else
	    statusBar()->message( "Loading aborted", 2000 );    
}


void bsMainWindow::loadSettings( const char *fileName ) {
    m_document->loadSettings(fileName);
    if (m_document->lastError() != 0){
	QMessageBox::information( this,"ERROR", m_document->lastMessage());
    } 
    statusBar()->message(m_document->lastMessage(), 6000 );
}


